%FLEXVEG%
%Written by Tim Marjoribanks and Maike Paul (t.i.marjoribanks@lboro.ac.uk)

%This routine calculates the equilibrium stem position for vegetation stems
%using a local rigidity model and is presented fully in the paper:

% Marjoribanks, T.I. and Paul, M. (2021) Modelling flow-induced 
% reconfiguration of variable rigidity aquatic vegetation, Journal of 
% Hydraulic Research, DOI:10.1080/00221686.2020.1866693 

%%%%%%%%%%%%%%%%%%%%%%     PARAMETERS     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ns          :number of vertical stem segments [-]
%   L           :total stem length [m]
%   stem_d_in   :*stem diameter [m]
%   stem_t_in   :*stem thickness [m] -value ignored for circular x-sections
%   shape       :integer number for corresponding x-section shapes
%   stem_rho    :*stem material density [kg/m^3]
%   Base_theta  :angle between ground and stem at base [rad]
%   stiff       :*Youngs Modulus (E) [Nm^2]
%   Uvel        :vertical profile of velocity values [ms^-1]
%   Load_in     :*distributed or point load vector in x-direction (per unit length)
%   init        :*initial curvature vector
%   Vel_inc     :vertical distance between velocity readings in Uvel [m]
%
%   *These variables may either be entered as a single value or a matrix of
%   sampled values with first column containing distance upstem and second 
%   column containing corresponding values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OUTPUTS:
%   x           :x coordinates of stem [m]
%   y           :y coordinates of stem [m]
%   theta       :angle at each stem node [rad]
%   sumfx       :Total drag force in x direction [N]
%   carterr     :Cartesian position error [m]
%   
%   (Other variables available for output, e.g. sumfy, t)
%
%  SHAPE CODES
%   1           :circle, only stem_d used, stem_t_in ignored
%   2           :rectangle/square, where d perpendicular to flow


function [x, y, theta, sumfx, sumfy carterr t]=flexvegJHR(ns,L,stem_d_in,stem_t_in,shape,stem_rho_in,Base_theta,E_in,Uvel,init,Load_in,Vel_inc)
tic
%Define section length
ds=L/ns;
nsvec=zeros(ns,1);
for i=1:ns
    nsvec(i)=i/ns;
end

%Get element-centered and hinge-centered values
%Interpolate Stem Diameter
if(size(stem_d_in,1)>1)
    stem_d=(interp1(stem_d_in(:,1),stem_d_in(:,2),(ds):ds:L,'linear','extrap'))';
    stem_d_hinge=(interp1(stem_d_in(:,1),stem_d_in(:,2),(ds/2):ds:L,'linear','extrap'))';
else
    stem_d=ones(ns,1).*stem_d_in;
    stem_d_hinge=stem_d;
end

%Interpolate stem curvature
if(size(init,1)>1)
    init_c=(interp1(init(:,1),init(:,2),(ds):ds:L,'linear','extrap'))';
    init_c_hinge=(interp1(init(:,1),init(:,2),(ds/2):ds:L,'linear','extrap'))';
else
    init_c=ones(ns,1).*init;
    init_c_hinge=init_c;
end

%Interpolate Stem Thickness
if(size(stem_t_in,1)>1)
    stem_t=(interp1(stem_t_in(:,1),stem_t_in(:,2),(ds):ds:L,'linear','extrap'))';
    stem_t_hinge=(interp1(stem_t_in(:,1),stem_t_in(:,2),(ds/2):ds:L,'linear','extrap'))';
else
    stem_t=ones(ns,1).*stem_t_in;
    stem_t_hinge=stem_t;
end

%Interpolate E
if(size(E_in,1)>1)
    E=(interp1(E_in(:,1),E_in(:,2),(ds):ds:L,'linear','extrap'))';
    E_hinge=(interp1(E_in(:,1),E_in(:,2),(ds/2):ds:L,'linear','extrap'))';
else
    E=ones(ns,1).*E_in;
    E_hinge=E;
end

%Calculate EI and area
switch shape
    case 1 %circles
        stiff=E_hinge.*(pi/4).*(stem_d_hinge/2).^4;
        area=pi.*(stem_d/2).^2;
    case 2 %rectangle
        stiff=E_hinge.*stem_d_hinge.*(stem_t_hinge.^3)./12;
        area=stem_d.*stem_t;
end

%Interpolate Stem Density
if(size(stem_rho_in,1)>1)
    stem_rho=(interp1(stem_rho_in(:,1),stem_rho_in(:,2),(ds):ds:L,'linear','extrap'))';
    stem_rho_hinge=(interp1(stem_rho_in(:,1),stem_rho_in(:,2),(ds/2):ds:L,'linear','extrap'))';
else
    stem_rho=ones(ns,1).*stem_rho_in;
    stem_rho_hinge=stem_rho;
end

%Interpolate Distributed Load
if(size(Load_in,1)>1)
    Load=(interp1(Load_in(:,1),Load_in(:,2),(ds):ds:L,'linear','extrap'))'; 
else
    Load=ones(ns,1).*Load_in;
end
Load
pause

%=================DEFINE FORCES===========================================%
%DRAG
buoy=9.81.*(998-stem_rho).*ds.*area;
buoy(ns)=9.81.*(998-stem_rho(ns)).*ds.*area(ns)/2;
forcez=buoy;
%switch off buoyancy if necessary
forcez=0;

theta=ones(ns,1)*Base_theta;
thetaold=Base_theta;
for i=1:ns
    theta(i)=thetaold-init_c_hinge(i)*ds;
    thetaold=theta(i);
end
theta0=zeros(ns,1);
theta0(1)=Base_theta;

%================START OF SOLUTION SEQUENCE===============================%
 H2=zeros(ns);
    for i=1:ns
        H2(i,min(i+1,ns))=-1;
        H2(i,i)=1;
    end
    
    H2inv=inv(H2);
%get initial value for xout
[xout]=solver(ns,Uvel,stem_d,theta,ds,stiff,forcez,theta0,init_c,shape,H2inv,Load,Vel_inc);
carterr=10000;
j=1;
%cycle through different relaxation values to assess optimum value
while(carterr>0.0001)
    olderror=10000000000000000000000;
    newerror=1000000000000000000000;
    carterr=0;
    factor=1;
    xoutstar=0;
    sumfx=0;
    while((abs(newerror)<abs(olderror))&&(factor>0.001))
        olderror=newerror;
        oldxoutstar=xoutstar;
        oldcarterr=carterr;
        sumfxold=sumfx;
        deltheta=(xout-theta);
        if abs(sum(deltheta))>(ns*pi/2)
            deltheta=deltheta*(ns*pi/2)/abs((sum(deltheta)));
        end
        thetastar=theta+factor*deltheta;
        [xoutstar, carterr, newerror, sumfx, sumfy]=solver(ns,Uvel,stem_d,thetastar,ds,stiff,0,theta0,init_c_hinge,shape,H2inv,Load,Vel_inc);
        factor=factor*0.1;
        xoutstar;     
    end
      
    deltheta=(xout-theta);
    %Limit change in one iteration
    if abs(sum(deltheta))>(ns*pi/2)
        deltheta=deltheta*(ns*pi/2)/abs((sum(deltheta)));
    end
    theta=theta+factor*100*deltheta;

    out_fact(j)=factor*100;
    out_err(j)=oldcarterr;
    
    carterr=oldcarterr
    xout=oldxoutstar;
    
    j=j+1;
    sumfx=sumfxold;
    
end
startx=0;
starty=0;
x=zeros(ns+1,1);
y=zeros(ns+1,1);
y(1)=ds*sin(theta0(1))/2;
x(1)=ds*cos(theta0(1))/2;
for i=1:ns
    if(i==1)
        x(i+1)=startx+ds*(cos(theta(i))+cos(theta0(1)))/2;
        y(i+1)=starty+ds*(sin(theta(i))+sin(theta0(1)))/2;
    else
        x(i+1)=startx+ds*(cos(theta(i))+cos(theta(i-1)))/2;
        y(i+1)=starty+ds*(sin(theta(i))+sin(theta(i-1)))/2;
    end
    startx=x(i+1);
    starty=y(i+1);       
end
toc;
t=toc;
end  

function [xout, carterr, angerr, sumfx, sumfy]=solver(ns,Uvel,stem_d,theta,ds,stiff,forcez,theta0,init_c,shape,H2inv,Load,Vel_inc)
dsmat=zeros(ns);
for i=1:ns
    dsmat(i,i)=ds;
end
dsmat(ns,ns)=ds/2;

startx=0;
starty=0;
x=zeros(ns+1,1);
y=zeros(ns+1,1);
y(1)=ds*sin(theta0(1))/2;
x(1)=ds*cos(theta0(1))/2;
for i=1:ns
    if(i==1)
        x(i+1)=startx+ds*(cos(theta(i))+cos(theta0(1)))/2;
        y(i+1)=starty+ds*(sin(theta(i))+sin(theta0(1)))/2;
    else
        x(i+1)=startx+ds*(cos(theta(i))+cos(theta(i-1)))/2;
        y(i+1)=starty+ds*(sin(theta(i))+sin(theta(i-1)))/2;
    end
    startx=x(i+1);
    starty=y(i+1);       
end


%Different end and distributed load settings
% Fend=zeros(ns,1);
% Fend(ns)=Load*abs(sin(theta(ns)));
% Fvert=zeros(ns,1);
% Fvert(ns)=Load*abs(sin(theta(ns)));
%forcex=ones(ns,1)*0.758*ds;
%forcex=ones(ns,1)*0.0*ds;
forcex=zeros(ns,1);

%forcex(ns)=forcex(ns)+Load;

    for i=1:ns
        velind=max(ceil((y(i+1)-(ds/2)*sin(theta(i)))/Vel_inc),1);
        locvel=Uvel(velind);
        if(i==ns)
            ds2=ds/2;
        else
            ds2=ds;
        end
        if shape==1
            forcex(i)=0.5*998*1*stem_d(i)*ds2.*sin(theta(i)).*locvel^2;
        else
            forcex(i)=0.5*998*1.95*stem_d(i)*ds2.*sin(theta(i)).*locvel^2;
        end
    end
    
    A=zeros(ns);
    for i=1:ns
        A(i,i)=-1;
        if(i<ns)
            A(i,i+1)=1;
        end
    end
    K=stiff/(ds);
    K2=zeros(ns);
    for i=1:ns
        K2(i,i)=K(i);
    end
    AA=A*K2*A';
    
    Fx=zeros(ns,1);
    Fz=zeros(ns,1);

    L2=zeros(ns);
    for i=1:ns
        L2(i,i)=ds/2;
        for j=(i+1):ns
            L2(i,j)=ds;
        end
    end
    
    D2=zeros(ns);
    for i=1:ns
        D2(i,i)=1;
        D2(i,min(i+1,ns))=1;
    end
    
    C=zeros(ns);
    C(1,1)=cos(0.5*(theta(1)+theta0(1)));
    for i=2:ns
        C(i,i)=cos(0.5*(theta(i)+theta(i-1)));
    end
    
    S=zeros(ns);
    S(1,1)=sin(0.5*(theta(1)+theta0(1)));
    for i=2:ns
        S(i,i)=sin(0.5*(theta(i)+theta(i-1)));
    end
    
    C2=zeros(ns);
    C2(1,1)=cos(0.5*(theta(1)-theta0(1)));
    for i=2:ns
        C2(i,i)=cos(0.5*(theta(i)-theta(i-1)));
    end
    
    S2=zeros(ns);
    S2(1,1)=sin(0.5*(theta(1)-theta0(1)));
    for i=2:ns
        S2(i,i)=sin(0.5*(theta(i)-theta(i-1)));
    end
    
    %Without Loading
    %Fx=forcex.*sin(theta).*abs(sin(theta));
    %Fz=forcez-forcex.*abs(sin(theta)).*cos(theta);
    
    Fx=forcex.*sin(theta).*abs(sin(theta))+Load*ds;
    Fz=forcez-forcex.*abs(sin(theta)).*cos(theta);
    
    AI=inv(A*S);
    F=0.5*dsmat*D2*C2*inv(A*S+A*C*AI*A*C)*(-A*C*AI*Fz+Fx)+0.5*ds*D2*S2*AI*(-Fz+A*C*inv(A*C*AI*A*C+A*S)*(-A*C*AI*Fz+Fx));

    B=F+K(1)*theta0+A*(K2*(init_c.*ds));
   
    xout=AA\B;
        
    %get final cartesian positions
    startx=0;
    starty=0;
    outx=zeros(ns+1,1);
    outy=zeros(ns+1,1);
    outy(1)=ds*sin(theta0(1))/2;
    outx(1)=ds*cos(theta0(1))/2;
    for i=1:ns
        if(i==1)
            outx(i+1)=startx+ds*(cos(xout(i))+cos(theta0(1)))/2;
            outy(i+1)=starty+ds*(sin(xout(i))+sin(theta0(1)))/2;
        else
            outx(i+1)=startx+ds*(cos(xout(i))+cos(xout(i-1)))/2;
            outy(i+1)=starty+ds*(sin(xout(i))+sin(xout(i-1)))/2;
        end
        startx=outx(i+1);
        starty=outy(i+1);       
    end
    
    %get error
    angerr=sum(abs(theta-xout));
    carterr=sqrt((outx(ns+1)-x(ns+1))^2+(outy(ns+1)-y(ns+1))^2);
    sumfx=(Fx);
    sumfy=(Fz);
end
    
